@@ -2,6 +2,7 @@
   Copyright  :  (C) 2012-2016, University of Twente,
                     2016-2017, Myrtle Software Ltd,
                     2021-2022, QBayLogic B.V.
+                    2022     , LUMI GUIDE FIETSDETECTIE B.V.
   License    :  BSD2 (see the file LICENSE)
   Maintainer :  QBayLogic B.V. <devops@qbaylogic.com>
 
@@ -23,7 +24,7 @@ import           Control.Exception               (throw)
 import           Control.Lens
   (use, (%=), _1, _2, element, (^?))
 import           Control.Monad                   (forM, (<=<))
-import           Control.Monad.State             (State, StateT (..), lift)
+import           Control.Monad.State             (State, StateT (..), lift, gets)
 import           Data.Bitraversable              (bitraverse)
 import           Data.Bool                       (bool)
 import           Data.Coerce                     (coerce)
@@ -54,7 +55,7 @@ import           Text.Read                       (readEither)
 import           Text.Trifecta.Result            hiding (Err)
 
 import           Clash.Backend
-  (Backend (..), Usage (..), AggressiveXOptBB(..))
+  (Backend (..), Usage (..), AggressiveXOptBB(..), RenderEnums(..))
 import           Clash.Netlist.BlackBox.Parser
 import           Clash.Netlist.BlackBox.Types
 import           Clash.Netlist.Types
@@ -68,16 +69,61 @@ import           Clash.Signal.Internal
 import           Clash.Util
 import qualified Clash.Util.Interpolate          as I
 
+import           Clash.Annotations.Primitive     (HDL(VHDL))
+
 inputHole :: Element -> Maybe Int
 inputHole = \case
-  Arg n         -> pure n
-  Lit n         -> pure n
-  Const n       -> pure n
-  Name n        -> pure n
-  Typ (Just n)  -> pure n
-  TypM (Just n) -> pure n
-  Err (Just n)  -> pure n
-  _             -> Nothing
+  Text _           -> Nothing
+  Component _      -> Nothing
+  Result           -> Nothing
+  Arg n            -> pure n
+  ArgGen _ n       -> pure n
+  Const n          -> pure n
+  Lit n            -> pure n
+  Name n           -> pure n
+  ToVar _ n        -> pure n
+  Sym _ _          -> Nothing
+  Typ nM           -> nM
+  TypM nM          -> nM
+  Err nM           -> nM
+  TypElem _        -> Nothing
+  CompName         -> Nothing
+  IncludeName _    -> Nothing
+  IndexType _      -> Nothing
+  Size _           -> Nothing
+  Length _         -> Nothing
+  Depth _          -> Nothing
+  MaxIndex _       -> Nothing
+  FilePath _       -> Nothing
+  Template _ _     -> Nothing
+  Gen _            -> Nothing
+  IF _ _ _         -> Nothing
+  And _            -> Nothing
+  IW64             -> Nothing
+  CmpLE _ _        -> Nothing
+  HdlSyn _         -> Nothing
+  BV _ _ _         -> Nothing
+  Sel _ _          -> Nothing
+  IsLit n          -> pure n
+  IsVar n          -> pure n
+  IsScalar n       -> pure n
+  IsActiveHigh n   -> pure n
+  Tag n            -> pure n
+  Period n         -> pure n
+  LongestPeriod    -> Nothing
+  ActiveEdge _ n   -> pure n
+  IsSync n         -> pure n
+  IsInitDefined n  -> pure n
+  IsActiveEnable n -> pure n
+  IsUndefined n    -> pure n
+  StrCmp _ n       -> pure n
+  OutputWireReg n  -> pure n
+  Vars n           -> pure n
+  GenSym _ _       -> Nothing
+  Repeat _ _       -> Nothing
+  DevNull _        -> Nothing
+  SigD _ nM        -> nM
+  CtxName          -> Nothing
 
 -- | Determine if the number of normal/literal/function inputs of a blackbox
 -- context at least matches the number of argument that is expected by the
@@ -430,13 +476,15 @@ renderElem b (Tag n) = do
 
 renderElem b (IF c t f) = do
   iw <- iwWidth
+  hdl <- gets hdlKind
   syn <- hdlSyn
+  enums <- renderEnums
   xOpt <- aggressiveXOptBB
-  let c' = check (coerce xOpt) iw syn c
+  let c' = check (coerce xOpt) iw hdl syn enums c
   if c' > 0 then renderTemplate b t else renderTemplate b f
   where
-    check :: Bool -> Int -> HdlSyn -> Element -> Int
-    check xOpt iw syn c' = case c' of
+    check :: Bool -> Int -> HDL -> HdlSyn -> RenderEnums -> Element -> Int
+    check xOpt iw hdl syn enums c' = case c' of
       (Size e)   -> typeSize (lineToType b [e])
       (Length e) -> case lineToType b [e] of
                        (Vector n _)              -> n
@@ -476,6 +524,15 @@ renderElem b (IF c t f) = do
                       Literal {}   -> 1
                       BlackBoxE {} -> 1
                       _            -> 0
+      (IsScalar n) -> let (_,ty,_) = bbInputs b !! n
+                          isScalar _ Bit          = 1
+                          isScalar _ Bool         = 1
+                          isScalar VHDL Integer   = 1
+                          isScalar VHDL (Sum _ _) = case enums of
+                                                       RenderEnums True  -> 1
+                                                       RenderEnums False -> 0
+                          isScalar _ _            = 0
+                      in isScalar hdl ty
 
       (IsUndefined n) ->
         let (e, _, _) = bbInputs b !! n
@@ -539,14 +596,14 @@ renderElem b (IF c t f) = do
                 | t1 == Text.pack t2 -> 1
                 | otherwise -> 0
               Nothing -> error $ $(curLoc) ++ "Expected a string literal: " ++ show e
-      (And es)   -> if all (/=0) (map (check xOpt iw syn) es)
+      (And es)   -> if all (/=0) (map (check xOpt iw hdl syn enums) es)
                        then 1
                        else 0
-      CmpLE e1 e2 -> if check xOpt iw syn e1 <= check xOpt iw syn e2
+      CmpLE e1 e2 -> if check xOpt iw hdl syn enums e1 <= check xOpt iw hdl syn enums e2
                         then 1
                         else 0
-      _ -> error $ $(curLoc) ++ "IF: condition must be: SIZE, LENGTH, IW64, LIT, ISLIT, or ISARG"
-
+      _ -> error $ $(curLoc) ++ "IF: condition must be: SIZE, LENGTH, LIT, DEPTH, IW64, VIVADO, OTHERSYN, ISVAR, ISLIT, ISUNDEFINED, ISACTIVEENABLE, ACTIVEEDGE, ISSYNC, ISINITDEFINED, ISACTIVEHIGH, STRCMP, AND, ISSCALAR or CMPLE."
+                             ++ "\nGot: " ++ show c'
 renderElem b e = fmap const (renderTag b e)
 
 parseFail :: Text -> BlackBoxTemplate
@@ -607,7 +664,7 @@ renderTag b (Result)    = do
   fmap renderOneLine . getAp . expr False . fst $ bbResult "~RESULT" b
 renderTag b (Arg n)  = do
   let (e,_,_) = bbInputs b !! n
-  renderOneLine <$> getAp (expr False e)
+  renderOneLine <$> getAp (expr True e)
 
 renderTag b (Const n)  = do
   let (e,_,_) = bbInputs b !! n
@@ -726,9 +783,9 @@ renderTag b (FilePath e)    = case e of
         return (Text.pack (show s'))
       _ -> do
         e2  <- getAp (prettyElem e)
-        error $ $(curLoc) ++ "argument of ~FILEPATH:" ++ show e2 ++  "does not reduce to a string"
+        error $ $(curLoc) ++ "argument of ~FILE:" ++ show e2 ++  "does not reduce to a string"
   _ -> do e' <- getAp (prettyElem e)
-          error $ $(curLoc) ++ "~FILEPATH expects a ~LIT[N] argument, but got: " ++ show e'
+          error $ $(curLoc) ++ "~FILE expects a ~LIT[N] argument, but got: " ++ show e'
 renderTag b (IncludeName n) = case indexMaybe (bbQsysIncName b) n of
   Just nm -> return (Text.fromStrict nm)
   _ -> error $ $(curLoc) ++ "~INCLUDENAME[" ++ show n ++ "] does not correspond to any index of the 'includes' field that is specified in the primitive definition"
@@ -933,6 +990,7 @@ prettyElem (Sel e i) = do
   renderOneLine <$> (string "~SEL" <> brackets (string e') <> brackets (int i))
 prettyElem (IsLit i) = renderOneLine <$> (string "~ISLIT" <> brackets (int i))
 prettyElem (IsVar i) = renderOneLine <$> (string "~ISVAR" <> brackets (int i))
+prettyElem (IsScalar i) = renderOneLine <$> (string "~ISSCALAR" <> brackets (int i))
 prettyElem (IsActiveHigh i) = renderOneLine <$> (string "~ISACTIVEHIGH" <> brackets (int i))
 prettyElem (IsActiveEnable i) = renderOneLine <$> (string "~ISACTIVEENABLE" <> brackets (int i))
 prettyElem (IsUndefined i) = renderOneLine <$> (string "~ISUNDEFINED" <> brackets (int i))
@@ -1039,6 +1097,7 @@ walkElement f el = maybeToList (f el) ++ walked
         Sel e _ -> go e
         IsLit _ -> []
         IsVar _ -> []
+        IsScalar _ -> []
         Tag _ -> []
         Period _ -> []
         LongestPeriod -> []
@@ -1123,6 +1182,7 @@ getUsedArguments (N.BBTemplate t) = nub (concatMap (walkElement matchArg) t)
         IndexType _ -> Nothing
         IsActiveHigh _ -> Nothing
         IsVar _ -> Nothing
+        IsScalar _ -> Nothing
         IW64 -> Nothing
         Length _ -> Nothing
         MaxIndex _ -> Nothing
